/*=========================================================================*/
/*                              LOCKMENU.C                                 */
/*                                                                         */
/*  Various menu functions used by IconLock.DLL                            */
/*                                                                         */
/*                                                                         */
/*                                                                         */
/*               IconLock Version 1.0 Copyright (c) 1999                   */
/*                         Ziff Davis, Inc.							       */
/*                       All rights reserved.							   */
/*        First appeared in PC Magazine, US Edition, August, 1999	       */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/
#include "common.h"
#include "lockdll.h"



//=====================================================================
//  FUNCTION:		SearchForMenuItem
//
//  PURPOSE:		Enumerates a menu for an item id
//					Note that if the menu has submenus
//					this function will call itself to enumerate
//                  the submenu -- i.e. its recursive
//
//  PARAMETERS:		handle to the menu and the ID to search for		
//
//  RETURN VALUE:	sets pID to zero if the menu item id was found
//
//  COMMENTS:		
//=====================================================================
void SearchForMenuItem( HMENU hMenu, UINT *pID )
{
	int i, iCount;
	MENUITEMINFO info;
	char szMenuItem[80];

	iCount = GetMenuItemCount( hMenu );

	for( i = 0; i < iCount && *pID; i++ )
	{
        info.cbSize = sizeof( MENUITEMINFO );
		info.cch = sizeof( szMenuItem ); 
		info.dwTypeData = (LPSTR)szMenuItem;    //cast it or fail@#$#$!!
		info.fMask = MIIM_DATA | MIIM_ID | MIIM_TYPE | MIIM_STATE | MIIM_SUBMENU;
        if( GetMenuItemInfo( hMenu, i, TRUE, &info ) )
		{
            if( info.wID == *pID )
            {
                *pID = 0;
            }

            if( info.hSubMenu && *pID )
                SearchForMenuItem( info.hSubMenu, pID );        
        }
    }

}

//=====================================================================
//  FUNCTION:		GetUniqueMenuID
//
//  PURPOSE:		Searchs a menu for a menu ID.  If it exists then
//                  the searched id is incremented.		
//
//  PARAMETERS:		
//
//  RETURN VALUE:	a unique menu id
//
//  COMMENTS:		
//=====================================================================
UINT GetUniqueMenuID( HMENU hMenu, UINT wStartID  )
{
    UINT wID = wStartID;

    while( TRUE )
    {
	    SearchForMenuItem( hMenu, &wID );
        if( wID == 0 )
            wID = ++wStartID;
        else 
            break;
    }
    return( wID );
}

//=====================================================================
//  FUNCTION:		SetLockMenuOptions
//
//  PURPOSE:		Parses our popup menu for the Quick Restore
//                  and Undo menu options and disables them if necessary.
//
//  PARAMETERS:		hMenu -- handle to popup menu
//
//  RETURN VALUE:	none
//
//  COMMENTS:       Note that the menu ID is stored in info.dwItemData
//=====================================================================
void SetLockMenuOptions( HMENU hMenu )
{
	int i, iCount;
	MENUITEMINFO info;
	char szMenuItem[80];

	BOOL bDisableResolution = ( HasResolutionBeenSaved( ) == FALSE );
	BOOL bDisableUndo = ( HasUndoBeenSaved( ) == FALSE );

	iCount = GetMenuItemCount( hMenu );

	for( i = 0; i < iCount; i++ )
	{
		info.cbSize = sizeof( MENUITEMINFO );
		info.cch = sizeof( szMenuItem ); 
		info.dwTypeData = (LPSTR)szMenuItem;    //cast it or fail
		info.fMask = MIIM_DATA | MIIM_ID | MIIM_TYPE | MIIM_STATE;
		
		if( GetMenuItemInfo( hMenu, i, TRUE, &info ) )
		{
			info.fState = MFS_DISABLED | MFS_GRAYED;

			if( bDisableResolution && info.dwItemData == IDD_RESTORE )
			 	SetMenuItemInfo( hMenu, i, TRUE, &info );

			if( bDisableUndo && info.dwItemData == IDD_UNDO )
			 	SetMenuItemInfo( hMenu, i, TRUE, &info );
		}
	}

}


//=====================================================================
//  FUNCTION:		DisableIconLockMenuItems
//
//  PURPOSE:		Parses our popup menu and disables all
//                  items except for the Unlock desktop option
//
//  PARAMETERS:		hMenu -- handle to popup menu
//
//  RETURN VALUE:	none
//
//  COMMENTS:       Only called if desktop is locked
//=====================================================================
void DisableIconLockMenuItems( HMENU hMenu )
{
	int i, iCount;
	MENUITEMINFO info;
	char szMenuItem[80];
			
	if( hMenu )
	{
		iCount = GetMenuItemCount( hMenu );

		for( i = 0; i < iCount; i++ )
		{
			info.cbSize = sizeof( MENUITEMINFO );
			info.cch = sizeof( szMenuItem ); 
			info.dwTypeData = (LPSTR)szMenuItem;		
			info.fMask = MIIM_DATA | MIIM_ID | MIIM_TYPE | MIIM_STATE;
			
			if( GetMenuItemInfo( hMenu, i, TRUE, &info ) )
			{
				if( info.dwItemData != IDD_LOCKDESKTOP && info.dwItemData != IDD_OPTIONS)
					info.fState = MFS_DISABLED | MFS_GRAYED;
				
				if( info.dwItemData == IDD_LOCKDESKTOP )
					lstrcpy( szMenuItem, SZ_UNLOCK );
				
				SetMenuItemInfo( hMenu, i, TRUE, &info );
			}
		}

	}
	
}

//=====================================================================
//  FUNCTION:		StripAmpersand
//
//  PURPOSE:		Removes the & from a menu string	
//
//  PARAMETERS:		
//
//  RETURN VALUE:	modfied string in pSource	
//
//  COMMENTS:		
//=====================================================================
void StripAmpersand( LPSTR pSource  )
{
	LPSTR pDest = pSource;

	strupr( pSource );
			
	while( *pSource )	
	{
		if( *pSource != '&' )
			*pDest++ = *pSource;
		pSource++;
	}

	*pDest = 0;

}

//=====================================================================
//  FUNCTION:		IsMenuStringInList
//
//  PURPOSE:		Searches the list of menu string to be disabled for the	
//					pMenuItem
//  PARAMETERS:		the string to search for
//
//  RETURN VALUE:	TRUE if it is to be disabled	
//
//  COMMENTS:		
//=====================================================================
BOOL IsMenuStringInList( LPCSTR pMenuItem ) 
{
int i;
LPSTR p;
char szDesktopItem[MAX_MENUSTRING_LEN];
char szMenuItem[MAX_MENUSTRING_LEN];

	
	//make a local copy
	strncpy( szDesktopItem, pMenuItem, sizeof( szDesktopItem ) );
	szDesktopItem[MAX_MENUSTRING_LEN-1] = 0;	//be sure that we are null terminated
	StripAmpersand( szDesktopItem );

	//don't let user disable the 'IconLock' menu option.
	if( strcmpi( szDesktopItem, SZ_APPNAME ) == 0 )
		return( FALSE );


	if( hMenuStrings )
	{
		PMENUSTRINGS pMenuStrings = (PMENUSTRINGS)hMenuStrings;
		for( i = 0; i < iconlock.iNumberOfMenuStrings; i++ )
		{
			if( pMenuStrings[i].szMenuString[0] )
			{
				strncpy( szMenuItem, pMenuStrings[i].szMenuString, sizeof( szMenuItem ) );
				szMenuItem[MAX_MENUSTRING_LEN-1] = 0;	//be sure that we are null terminated
				StripAmpersand( szMenuItem );

				
				if( strcmpi( szDesktopItem, szMenuItem ) == 0 )
					return( TRUE );
	
				//check for wildcards
				if( p = strchr( szMenuItem, '*' ) )
				{
					*p = 0;
					 if( strnicmp( szDesktopItem, szMenuItem, strlen( szMenuItem ) ) == 0 )
						return( TRUE );
				}

			}

		}
		
	}

	return( FALSE );
}


//=====================================================================
//  FUNCTION:		DisableDesktopMenuItems
//
//  PURPOSE:		Parses the Desktop context menu disables all
//                  items.
//
//  PARAMETERS:		hMenu -- handle to Desktop popup menu
//
//  RETURN VALUE:	none
//
//  COMMENTS:       Only called if desktop is locked
//=====================================================================
void DisableDesktopMenuItems( HMENU hMenu )
{
	int i, iCount;
	MENUITEMINFO info;
	char szMenuItem[80];

	iCount = GetMenuItemCount( hMenu );
	for( i = 0; i < iCount; i++ )
	{
		info.cbSize = sizeof( MENUITEMINFO );
		info.cch = sizeof( szMenuItem ); 
		info.dwTypeData = (LPSTR)szMenuItem;		
		info.fMask = MIIM_DATA | MIIM_ID | MIIM_TYPE | MIIM_STATE;
		if( GetMenuItemInfo( hMenu, i, TRUE, &info ) )
		{
			if( info.fType == MFT_STRING )
			{						
				if( IsMenuStringInList( szMenuItem ) )
					EnableMenuItem( hMenu, i, MF_BYPOSITION | MF_GRAYED | MF_DISABLED );
			}
		}
	}
	
}


//=====================================================================
//  FUNCTION:		CopyMenu
//
//  PURPOSE:		Creates a temporary popup menu that will be appended
//                  to the Desktop context menu.   All menu items from 
//                  our resource menu are copied onto this menu.
//
//  PARAMETERS:		none
//
//  RETURN VALUE:	Handle to the new menu.
//
//  COMMENTS:       
//=====================================================================
HMENU CopyMenu( HMENU hDesktopMenu )	
{
	HMENU hNewMenu, hMenu, hSubMenu;
	int i,iCount;
	MENUITEMINFO info;
	char szMenuItem[80];

    hMenu = LoadMenu( hInstGlobal, MAKEINTRESOURCE( IDR_ICONLOCK ) );
	
    hNewMenu = CreatePopupMenu( );

	if( hMenu )
	{
		hSubMenu = GetSubMenu( hMenu, 0 );
		iCount = GetMenuItemCount( hSubMenu );

		//append our popup menu to the desktop menu
        AppendMenu( hDesktopMenu, MF_SEPARATOR,	0, 0 );
		GetMenuString( hMenu, 0, szMenuItem, sizeof( szMenuItem ), MF_BYPOSITION );
		AppendMenu( hDesktopMenu, MF_STRING | MF_POPUP,	(UINT)hNewMenu, szMenuItem );    
		
		for( i = 0; i < iCount; i++ )
		{
			info.cbSize = sizeof( MENUITEMINFO );
			info.fMask = MIIM_DATA | MIIM_ID | MIIM_TYPE;
			info.dwTypeData = (LPSTR)szMenuItem;
			info.cch = sizeof( szMenuItem ); 

			GetMenuItemInfo( hSubMenu, i, TRUE, &info );

   			if( info.fType == MFT_STRING )
			{
				info.dwItemData  = info.wID;	//save the menu id in itemdata
				info.wID = GetUniqueMenuID( hDesktopMenu, info.wID );   //assign unique wID
				SetMenuItemInfo( hSubMenu , i, TRUE, &info );	//update the menu
			}

			InsertMenuItem( hNewMenu, i, TRUE, &info );
		}
    
		DestroyMenu( hMenu );//1.1
	}

	return( hNewMenu );
}


//=====================================================================
//  FUNCTION:		HasUndoBeenSaved
//
//  PURPOSE:		Searches ICONLOCK.INI for the Undo section		
//
//  PARAMETERS:		
//
//  RETURN VALUE:	TRUE if found
//
//  COMMENTS:		
//=====================================================================
BOOL HasUndoBeenSaved( )
{
	char szTmp[10];
	return( MyGetPrivateProfileString( SZ_UNDO, NULL, "", szTmp, sizeof(szTmp), szIniFile ) );
}




//=====================================================================
//  FUNCTION:		 HasResolutionBeenSaved
//
//  PURPOSE:		Gets the current dimensions of the display and
//                  checks to see if a resolution has been saved
//                  for the dimensions.
//
//  PARAMETERS:		none
//
//  RETURN VALUE:	returns TRUE if an INI entry exists for the resolution
//
//  COMMENTS:       Used to enable / disable the Quick Restore menu option		
//=====================================================================
BOOL HasResolutionBeenSaved( )
{
	char szTmp[10],szSection[80];
	
	GetScreenDimensions( szSection );
	return( MyGetPrivateProfileString( szSection, NULL, "", szTmp, sizeof(szTmp), szIniFile ) );
}



